@@ -10,7 +10,7 @@ import subprocess
 
 from dataclasses import dataclass, field
 
-import thingy.git2 as git
+import thingy.git as git
 import thingy.colour as colour
 
 ################################################################################
@@ -317,6 +317,8 @@ def add_new_repo(args, config, repo):
 
     abs_repo_path = absolute_repo_path(args, repo)
 
+    added = repo not in config
+
     config[repo] = {}
 
     default_branch = git.branch(path=abs_repo_path)
@@ -335,7 +337,10 @@ def add_new_repo(args, config, repo):
         config[repo]['repo name'] = os.path.basename(repo)
 
     if not args.quiet:
-        colour.write(f'Added [BLUE:{repo}] with default branch [BLUE:{default_branch}]')
+        if added:
+            colour.write(f'Added [BLUE:{repo}] with default branch [BLUE:{default_branch}]')
+        else:
+            colour.write(f'Reset [BLUE:{repo}] with default branch [BLUE:{default_branch}]')
 
 ################################################################################
 
@@ -362,14 +367,14 @@ def mg_clone(args, config, console):
         if os.path.isdir(directory):
             colour.error(f'The "[BLUE:{directory}]" directory already exists', prefix=True)
         else:
-            colour.error(f'[BLUE:{directory}]" already exists', prefix=True)
+            colour.error(f'"[BLUE:{directory}]" already exists', prefix=True)
 
     # Clone the repo and chdir into it
 
     if not args.quiet:
         colour.write(f'Cloning [BOLD:{args.parameters[0]}] into [BLUE:{directory}]')
 
-    git.clone(args.parameters[0], path=directory)
+    git.clone(args.parameters[0], working_tree=directory)
 
     os.chdir(directory)
 
@@ -401,7 +406,7 @@ def mg_clone(args, config, console):
             if not args.quiet:
                 colour.write(f'Cloning [BLUE:{repo["origin"]}] into [BLUE:{directory}]')
 
-            git.clone(repo['origin'], path=repo.name)
+            git.clone(repo['origin'], working_tree=repo.name)
 
             if not args.quiet:
                 colour.write(f'    Checking out [BLUE:{repo["default branch"]}]')
@@ -445,7 +450,7 @@ def mg_init(args, config, console):
 
     for entry in removals:
         del config[entry]
-        colour.write(f'Removed [BLUE:{repo}] as it no longer exists')
+        colour.write(f'Removed [BLUE:{entry}] as it no longer exists')
 
     # The configuration file needs to be updated
 
@@ -472,9 +477,21 @@ def mg_add(args, config, console):
     location = args.parameters[1]
 
     if os.path.exists(location):
-        colour.error(f'"[BLUE:{location}]" already exists', prefix=True)
+        if os.path.isdir(os.path.join(location, '.git')):
+            remotes = git.remotes(path=location)
 
-    git.clone(repo, path=location)
+            for remote in remotes:
+                if repo.endswith('.git') and repo == remotes[remote]:
+                    break
+                elif f'{repo}.git' == remotes[remote]:
+                    break
+            else:
+                colour.error(f'"[BLUE:{location}]" already exists and was not cloned from [BLUE:{repo}]', prefix=True)
+        else:
+            colour.error(f'"[BLUE:{location}]" already exists and is not a Git working tree', prefix=True)
+
+    else:
+        git.clone(repo, working_tree=location)
 
     # Add to the configuration
 
@@ -507,41 +524,32 @@ def mg_dir(args, config, console):
     # If a parameter is specified, look for matches, otherwise just return the location of the
     # configuration file
 
-    if args.parameters:
-        location = []
-        wild_location = []
+    if not args.parameters:
+        colour.write(os.path.dirname(args.configuration_file))
+        return
 
-        search_name = args.parameters[0]
+    locations = []
 
-        # Search for exact matches, or matches that contain the search term if it
-        # doesn't already contain a wildcard
+    search_name = args.parameters[0]
 
+    # Search for wildcard matches, or matches that contain the search term if it
+    # doesn't already contain a wildcard
+
+    if '*' in search_name or '?' in search_name:
+        search_name = f'*{search_name}*'
         for repo in select_git_repos(args, config):
             if fnmatch.fnmatch(repo['repo name'], search_name):
-                location.append(repo.name)
-
-            elif '*' not in search_name:
-                if fnmatch.fnmatch(repo['repo name'], f'*{search_name}*'):
-                    wild_location.append(repo.name)
-
-        # Look for a single exact match, a prefix with '*' match or prefix+suffix
-
-        destination = None
-        for destinations in (location, wild_location):
-            if len(destinations) == 1:
-                destination = destinations
-                break
-
-            if len(destinations) > 1:
-                destination = destinations
-
-        if not destination:
-            colour.error(f'No matches with [BLUE:{search_name}]', prefix=True)
-
-        colour.write("\n".join([relative_repo_path(args, d) for d in destination]))
-
+                locations.append(repo.name)
     else:
-        colour.write(os.path.dirname(args.configuration_file))
+        for repo in select_git_repos(args, config):
+            if search_name in repo['repo name']:
+                locations.append(repo.name)
+
+    if not locations:
+        colour.error(f'No matches with [BLUE:{search_name}]', prefix=True)
+
+    colour.write("\n".join([relative_repo_path(args, loc) for loc in locations]))
+
 
 ################################################################################
 
@@ -620,7 +628,7 @@ def mg_run(args, config, console):
     _ = console
 
     if not args.parameters:
-        colour.error('[BOLD:+run] command - missing parameter(s)')
+        colour.error('[BOLD:+run] command - missing parameter(s)', prefix=True)
 
     # Run the command in each of the working trees
 
